/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/gpl-3.0.txt
 *
 */
package com.frojasg1.textcrypt.application;

import com.frojasg1.general.desktop.dialogs.implementation.DesktopDialogsWrapper;
import com.frojasg1.general.desktop.dialogs.implementation.StaticDesktopDialogsWrapper;
import com.frojasg1.general.desktop.generic.DesktopGenericFunctions;
import com.frojasg1.general.exceptions.ConfigurationException;
import com.frojasg1.general.desktop.view.ToolTipLookAndFeel;
import com.frojasg1.general.dialogs.DialogsWrapper;
import com.frojasg1.general.progress.OperationCancellation;
import com.frojasg1.textcrypt.application.configuration.ApplicationConfiguration;
import com.frojasg1.textcrypt.controller.TextcryptController;
import com.frojasg1.textcrypt.controller.TextcryptView;
import com.frojasg1.textcrypt.controller.implementation.TextcryptController_imp;
import com.frojasg1.textcrypt.view.TextComponentJPanel;
import com.frojasg1.textcrypt.view.TextInfoJPanel;
import com.hotmail.frojasg1.applications.common.components.internationalization.window.InternationalizedJFrame;
import com.hotmail.frojasg1.applications.common.components.resizecomp.MapResizeRelocateComponentItem;
import com.hotmail.frojasg1.applications.common.components.resizecomp.ResizeRelocateItem;
import com.hotmail.frojasg1.encrypting.randomnumbers.RandomSource;
import java.awt.Component;
import java.awt.print.PrinterException;
import java.util.Vector;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;

/**
 *
 * @author axpe
 */
public class TextCryptJFrame extends InternationalizedJFrame
		implements TextcryptView
{
	protected static final String sa_configurationBaseFileName = "textCryptJFrame";

	protected TextcryptController _controller = null;
	protected TextComponentJPanel _textComponent = null;
	protected TextInfoJPanel _textInfo = null;

	protected DialogsWrapper _dw = null;

	/**
	 * Creates new form textCryptJFrame
	 */
	public TextCryptJFrame() throws ConfigurationException
	{
		super( ApplicationConfiguration.instance() );

		initConfiguration();

		initComponents();

		initOwnComponents();

		setWindowConfiguration();

		RandomSource.instance();		// it starts getting random bytes.
	}

	protected void initConfiguration() throws ConfigurationException
	{
		try
		{
			getAppliConf().M_openConfiguration();
		}
		catch( ConfigurationException ce )
		{
			ce.printStackTrace();

			StaticDesktopDialogsWrapper.showMessageDialog(null, ce.getMessage() + " Exiting from application",
											"Configuration error", JOptionPane.ERROR_MESSAGE, null );
			throw( ce );
		}

		DesktopGenericFunctions.createInstance( ApplicationConfiguration.instance() );
	}

	protected void initOwnComponents()
	{
		_dw = DesktopDialogsWrapper.createInstance( getAppliConf() );

		_controller = new TextcryptController_imp( this, _dw );

		_textInfo = new TextInfoJPanel( null, _controller );
		jPanel1.add( _textInfo );
/*		_textInfo.setBounds( 0, 0,
							(int) _textInfo.getPreferredSize().getWidth(),
							(int) _textInfo.getPreferredSize().getHeight() );
*/
		_textInfo.setBounds( 0, 0,
							(int) jPanel1.getWidth() - 5,
							(int) _textInfo.getPreferredSize().getHeight() );

		_textComponent = new TextComponentJPanel( _controller );
		jPanel1.add( _textComponent );
/*		_textComponent.setBounds( 0, (int) _textInfo.getHeight(),
							(int) _textComponent.getPreferredSize().getWidth(),
							(int) _textComponent.getPreferredSize().getHeight() );
*/
		_textComponent.setBounds( 8, (int) _textInfo.getHeight(),
							(int) jPanel1.getWidth() - 16,
							(int) ( jPanel1.getHeight() - _textInfo.getHeight() - 8 ) );

		pack();
	}

	protected void setWindowConfiguration( )
	{
		Vector<JPopupMenu> vectorJpopupMenus = null;

		MapResizeRelocateComponentItem mapRRCI = new MapResizeRelocateComponentItem( );
		try
		{
			mapRRCI.putResizeRelocateComponentItem(jPanel1, ResizeRelocateItem.RESIZE_TO_RIGHT +
															ResizeRelocateItem.RESIZE_TO_BOTTOM );
			mapRRCI.putResizeRelocateComponentItem(_textInfo, ResizeRelocateItem.RESIZE_TO_RIGHT );
			mapRRCI.putAll( _textInfo.getResizeRelocateInfo() );
			mapRRCI.putResizeRelocateComponentItem(_textComponent, ResizeRelocateItem.RESIZE_TO_RIGHT +
															ResizeRelocateItem.RESIZE_TO_BOTTOM );
			mapRRCI.putAll( _textComponent.getResizeRelocateInfo() );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}
/*
		a_intern = new JFrameInternationalization(	ApplicationConfiguration.sa_MAIN_FOLDER,
													ApplicationConfiguration.sa_APPLICATION_NAME,
													ApplicationConfiguration.sa_CONFIGURATION_GROUP,
													ApplicationConfiguration.sa_PATH_PROPERTIES_IN_JAR,
													a_configurationBaseFileName,
													this,
													parent,
													a_vectorJpopupMenus,
													false,
													mapRRCI );
*/
		createInternationalization(	getAppliConf().getConfigurationMainFolder(),
									getAppliConf().getApplicationNameFolder(),
									getAppliConf().getApplicationGroup(),
									getAppliConf().getInternationalPropertiesPathInJar(),
									sa_configurationBaseFileName,
									this,
									null,
									vectorJpopupMenus,
									true,
									mapRRCI );

//		setIcon( "com/frojasg1/chesspdfbrowser/resources/icons/App.icon.png" );
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("Textcrypt.v1.3");
        setMinimumSize(new java.awt.Dimension(715, 380));
        setName(""); // NOI18N
        getContentPane().setLayout(null);

        jPanel1.setLayout(null);
        getContentPane().add(jPanel1);
        jPanel1.setBounds(0, 0, 700, 340);

        pack();
    }// </editor-fold>//GEN-END:initComponents

	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[]) {
		/* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
		 * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
		 */
		try {
/*
			for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
				if ("Nimbus".equals(info.getName())) {
					javax.swing.UIManager.setLookAndFeel(info.getClassName());
					break;
				}
			}
*/
			javax.swing.UIManager.setLookAndFeel( new ToolTipLookAndFeel() );
/*		} catch (ClassNotFoundException ex) {
			java.util.logging.Logger.getLogger(TextCryptJFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (InstantiationException ex) {
			java.util.logging.Logger.getLogger(TextCryptJFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (IllegalAccessException ex) {
			java.util.logging.Logger.getLogger(TextCryptJFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
*/		} catch (javax.swing.UnsupportedLookAndFeelException ex) {
			java.util.logging.Logger.getLogger(TextCryptJFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		}
        //</editor-fold>
        //</editor-fold>

		/* Create and display the form */
		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
				try
				{
					new TextCryptJFrame().setVisible(true);
				}
				catch( Throwable th )
				{
					th.printStackTrace();
				}
			}
		});
	}

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel jPanel1;
    // End of variables declaration//GEN-END:variables

	@Override
	public String textCrypt_getFileName()
	{
		return( _textInfo.getFileName() );
	}

	@Override
	public String textCrypt_getText()
	{
		return( _textComponent.getText() );
	}

	@Override
	public boolean textCrypt_arePasswordEqual()
	{
		return( _textInfo.M_arePasswordsEqual() );
	}

	@Override
	public char[] textCrypt_getPassword()
	{
		char[] result = null;

		if( textCrypt_arePasswordEqual() )
			result = _textInfo.getPassword();

		return( result );
	}

	@Override
	public void textCrypt_setText(String text)
	{
		_textComponent.setText( text );
	}

	@Override
	public void textCrypt_setOperationCancellation(OperationCancellation oc)
	{
		_textInfo.setOperationCancellation(oc);
	}

	@Override
	public void textCrypt_printText() throws PrinterException
	{
		_textComponent.print();
	}

	@Override
	public void textCrypt_blockOpenAndSave()
	{
		_textInfo.blockOpenAndSave();
	}

	@Override
	public void textCrypt_unblockOpenAndSave()
	{
		_textInfo.unblockOpenAndSave();
	}

	@Override
	public int textCrypt_getStrength()
	{
		return( _textInfo.getStrength() );
	}

	@Override
	public void textCrypt_setStrength( int strength )
	{
		_textInfo.setStrength( strength );
	}

	@Override
	public void textCrypt_setFileName( String fileName )
	{
		_textInfo.setFileName( fileName );
	}

	@Override
	public void up_childStarts()
	{
		_textInfo.up_childStarts();
		repaint();
	}

	@Override
	public void up_updateProgressFromChild(double completedFraction)
	{
		_textInfo.up_updateProgressFromChild( completedFraction );
		repaint();
	}

	@Override
	public void up_childEnds()
	{
		_textInfo.up_childEnds();
		repaint();
	}

	@Override
	public OperationCancellation up_getOperationCancellation()
	{
		return( _textInfo.up_getOperationCancellation() );
	}

	@Override
	public void up_setDebug( boolean debug )
	{
		_textInfo.up_setDebug( debug );
	}

	@Override
	public ApplicationConfiguration getAppliConf()
	{
		return( (ApplicationConfiguration) super.getAppliConf() );
	}

	@Override
	public void formWindowClosing( boolean close )
	{
		if( close )
			RandomSource.M_clearInstance();

		super.formWindowClosing( close );
	}

	@Override
	public Component getComponent()
	{
		return( this );
	}

	@Override
	public void textCrypt_closeWindow()
	{
		formWindowClosing( true );
	}

	@Override
	public void formWindowClosingEvent( )
	{
		_controller.textCrypt_closing();
		// if it can be closed, _controller will call textCrypt_closeWindow()
	}

	@Override
	public void textCrypt_invokeLaterResetTextModified()
	{
		SwingUtilities.invokeLater( new Runnable() {
				@Override
				public void run()
				{
					_controller.textCrypt_resetTextModified();
				}
		});
	}

}
