/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.hotmail.frojasg1.encrypting.pseudorandomgenerator.chaoticgenerator;


import com.frojasg1.general.GeneralException;
import com.frojasg1.general.HexadecimalFunctions;
import com.hotmail.frojasg1.encrypting.encoderdecoder.ReordererEncoder;
import com.hotmail.frojasg1.encrypting.pseudorandomgenerator.PseudoRandomGenerator;
import com.hotmail.frojasg1.encrypting.pseudorandomgenerator.PseudoRandomGeneratorException;
import com.hotmail.frojasg1.encrypting.randomnumbers.RandomSource;
import java.math.MathContext;
import java.util.Date;

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 *
 * @author Usuario
 */
public class ChaoticGenerator_BigLong extends ChaoticGenerator_BigLong_depretated
{
	public ChaoticGenerator_BigLong()
	{
		super();
	}

	public ChaoticGenerator_BigLong( int sizeOfNumbers, int numberOfBitsPerIteration ) throws PseudoRandomGeneratorException
	{
		super( sizeOfNumbers, numberOfBitsPerIteration );
	}

	public long M_nextPartOfByte() throws PseudoRandomGeneratorException
	{
//		double xx_ini = a_xx.M_toDouble();
		a_xx = BigLong.M_square(a_xx);

//		double xx_square = a_xx.M_toDouble();
//		System.out.println( String.format( "BigLong: %.20g * %.20g = %.20g", xx_ini, xx_ini, xx_square ) );
//		System.out.println( String.format( "double: %.20g * %.20g = %.20g\n", xx_ini, xx_ini, xx_ini*xx_ini ) );
		a_xx = BigLong.M_add( a_xx, a_kk );
//		System.out.println( String.format( "BigLong: %.20g + %.20g = %.20g", xx_square, a_kk.M_toDouble(), a_xx.M_toDouble() ) );
//		System.out.println( String.format( "double: %.20g + %.20g = %.20g\n", xx_square, a_kk.M_toDouble(), (xx_square+a_kk.M_toDouble()) ) );

//		System.out.println( "a_xx: " + a_xx );
		long result = 0;
		if( a_numberOfBitsPerIteration < 8 )
		{
			int ones = a_xx.M_countOnes();
			result = (ones & a_maskForIteration);	// it fits in one byte
		}
		else
		{
			result = a_xx.M_getBytesInLong( a_numberOfBytesPerIteration );
		}
		return( result );
	}

	protected static void test_1()
	{
		try
		{
			Date time1 = new Date();
			
			PseudoRandomGenerator chg_1 = new ChaoticGenerator_BigLong( 6, 4 );
			PseudoRandomGenerator chg_2 = new ChaoticGenerator_BigLong( 6, 4 );
			RandomSource rs = RandomSource.instance();

			ReordererEncoder rd = new ReordererEncoder( chg_1, chg_2 );
			
			byte[] randomKey = rs.M_getRandomBytes( rd.M_getRecommendedNumberOfBytesToInitializeKey() );
			
			System.out.println( "random key:" );
			System.out.println( HexadecimalFunctions.M_getLogFromBuffer(randomKey) );
			
			rd.M_initializeKey(randomKey );
			
			byte[] buffer = new byte[256*1024];
			
			byte[] result = rd.M_encode(buffer, null);
			
			Date time2 = new Date();
			System.out.println( "time in ms " + (time2.getTime() - time1.getTime() )  );
/*
			int[] histogram = new int[16];
			for( int ii=0; ii<16; ii++ )	histogram[ii]=0;
			
			for( int ii=0; ii<8; ii++ )
			{
				for( int jj=0; jj<8; jj++ )
				{
					histogram[ chg.M_nextPartOfByte() ]++;
//					String hexStr = "00" + Integer.toHexString( chg.M_next() );
//					System.out.print( " " + hexStr.substring( hexStr.length()-2, hexStr.length()) );
				}
				
//				System.out.println("");
			}
			Date time2 = new Date();
			
			int ones = 0;
			int zeroes = 0;
			
			for( byte ii=0; ii<16; ii++ )
			{
				int onesByte = HexadecimalFunctions.M_countOnes(ii);
				System.out.println( "ones(ii) = "  + onesByte + ", histogram[" + ii + "] = " + histogram[ii] );
				zeroes += (4-onesByte) * histogram[ii];
				ones += onesByte * histogram[ii];
			}

			System.out.println( "zeroes: " + zeroes + " ones: " + ones );
			System.out.println( "time in ms " + (time2.getTime() - time1.getTime() )  );
*/
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}
	}
	

	protected static void test_2()
	{
		try
		{
			PseudoRandomGenerator chg_1 = new ChaoticGenerator_BigLong( 6, 4 );
			PseudoRandomGenerator chg_2 = new ChaoticGenerator_BigLong( 6, 4 );

			byte[] byteArrayKey = new byte[2];
			byteArrayKey[0]=117;
			byteArrayKey[1]=0;
			
			ReordererEncoder rd = new ReordererEncoder( chg_1, chg_2 );
			
			rd.M_initializeKey( byteArrayKey );
			
		}
		catch( PseudoRandomGeneratorException prge )
		{
			prge.printStackTrace();
		}
	}
	
	protected static void test_3()
	{
		try
		{
			String hexKey = "00112233";
			byte[] byteKey = HexadecimalFunctions.M_convertHexadecimalStringToByteArray(hexKey);
			System.out.println( "Cadena hex original: " + hexKey + ". Cadena hex recalculada: " +
								HexadecimalFunctions.M_convertByteArrayToHexadecimalString( byteKey ) );
		}
		catch( GeneralException ex )
		{
			ex.printStackTrace();
		}

		try
		{
			PseudoRandomGenerator chg_1 = new ChaoticGenerator_BigLong( 4, 2 );

			String key = "0000000000000000000000000012345678901234567890123456789012";
			chg_1.M_initialize( key );
			
		}
		catch( PseudoRandomGeneratorException prge )
		{
			prge.printStackTrace();
		}
	}
	
	
	public static void main( String[] params )
	{
		test_3();
	}
}
