/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.common.configuration.application;

import com.frojasg1.applications.common.configuration.ConfigurationParent;
import com.frojasg1.general.exceptions.ConfigurationException;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;


/**
 *
 * @author Usuario
 */
public abstract class BaseApplicationConfiguration extends ConfigurationParent implements BaseApplicationConfigurationInterface
{
	public static final String CONF_LAST_DIRECTORY = "LAST_DIRECTORY";
	public static final String CONF_LANGUAGE = "LANGUAGE";
//	public static final String CONF_ADDITIONAL_LANGUAGE = "ADDITIONAL_LANGUAGE";
	public static final String CONF_APPLICATION_FONT_FACTOR = "APPLICATION_FONT_FACTOR";
	public static final String CONF_FILE_CHOOSER_X = "FILE_CHOOSER_X";
	public static final String CONF_FILE_CHOOSER_Y = "FILE_CHOOSER_Y";
	public static final String CONF_FILE_CHOOSER_WIDTH = "FILE_CHOOSER_WIDTH";
	public static final String CONF_FILE_CHOOSER_HEIGHT = "FILE_CHOOSER_HEIGHT";

	public static final String ES_LANGUAGE = "ES";	 // Spanish language
	public static final String EN_LANGUAGE = "EN";	 // English language

	public BaseApplicationConfiguration( String mainFolder, String applicationName, String group,
								String language, String configurationFileName )
	{
		super( mainFolder, applicationName, group, language, configurationFileName );
	}

	@Override
	public Properties M_getDefaultProperties( String language )
	{
		Properties result = new Properties();

		result.setProperty(CONF_LANGUAGE, EN_LANGUAGE );
		result.setProperty(CONF_LAST_DIRECTORY, "" );

		result.setProperty(CONF_APPLICATION_FONT_FACTOR, "1.0" );

		return( result );
	}

	@Override
	public float getFontSizeFactor()
	{
		String factorStr = M_getStrParamConfiguration( CONF_APPLICATION_FONT_FACTOR );
		float factor = 1.0F;
		try
		{
			factor = Float.valueOf( factorStr );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}
		return( factor );
	}
	
	@Override
	public void setFontSizeFactor( float factor )
	{
		M_setFloatParamConfiguration( CONF_APPLICATION_FONT_FACTOR, factor );
	}

	@Override
	public String getLanguage()
	{
		return( M_getStrParamConfiguration( CONF_LANGUAGE ) );
	}

	@Override
	public void setLanguage( String language )
	{
		M_setStrParamConfiguration( CONF_LANGUAGE, language );
	}

	@Override
	public String getLastDirectory()
	{
		return( M_getStrParamConfiguration( CONF_LAST_DIRECTORY ) );
	}

	@Override
	public Rectangle getLastFileChooserBounds()
	{
		Rectangle result = null;
		
		int x = M_getIntParamConfiguration( CONF_FILE_CHOOSER_X );
		int y = M_getIntParamConfiguration( CONF_FILE_CHOOSER_Y );
		int width = M_getIntParamConfiguration( CONF_FILE_CHOOSER_WIDTH );
		int height = M_getIntParamConfiguration( CONF_FILE_CHOOSER_HEIGHT );

		if( (x>=0) && (y>=0) && (width>0) && (height>0) )
		{
			result = new Rectangle( x, y, width, height );
		}

		return( result );
	}

	@Override
	public void setLastDirectory( String directory )
	{
		M_setStrParamConfiguration( CONF_LAST_DIRECTORY, directory );
	}

	@Override
	public void setLastFileChooserBounds( Rectangle newBounds )
	{
		if( newBounds != null )
		{
			M_setIntParamConfiguration( CONF_FILE_CHOOSER_X, (int) newBounds.getX() );
			M_setIntParamConfiguration( CONF_FILE_CHOOSER_Y, (int) newBounds.getY() );
			M_setIntParamConfiguration( CONF_FILE_CHOOSER_WIDTH, (int) newBounds.getWidth() );
			M_setIntParamConfiguration( CONF_FILE_CHOOSER_HEIGHT, (int) newBounds.getHeight() );
		}
	}

	@Override
	public abstract String getConfigurationMainFolder();
	@Override
	public abstract String getApplicationNameFolder();
	@Override
	public abstract String getApplicationGroup();

	@Override
	public abstract String getInternationalPropertiesPathInJar();

	@Override
	public abstract String getAdditionalLanguage();

	@Override
	public abstract void setAdditionalLanguage( String language );

	@Override
	public abstract boolean getLicensesHaveBeenAccepted();

	@Override
	public abstract void setLicencesHaveBeenAccepted( boolean value );

	@Override
	public abstract String getDefaultLanguageConfigurationFolder( String language );

	@Override
	public abstract String getDefaultLanguage();

	@Override
	protected void changeLanguage_internal_simple( String language ) throws ConfigurationException
	{
		// this method is called from the parent. It loads the version of the configuration file for the new language
		// in the case of BaseApplicationConfiguration, it has not to be reloaded.
	}
}
