/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.hotmail.frojasg1.applications.common.components.resizecomp;

import com.hotmail.frojasg1.applications.common.components.internationalization.InternException;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.JSplitPane;
import javax.swing.plaf.SplitPaneUI;
import javax.swing.plaf.basic.BasicSplitPaneUI;

/**
 *
 * @author Usuario
 */
public class ResizeRelocateItem_JSplitPane extends ResizeRelocateItem implements MouseListener
{
	protected int _lastDraggedDividerLocation = -1;
	protected Dimension _jspSizeOfLastDraggedDividerLocationEvent = null;
	
	protected int _newDividerLocation = -1;
	protected JSplitPane _jsp = null;
	protected boolean _inDrag = false;
	
	protected ResizeRelocateItem_JSplitPane( JSplitPane comp, int flags, ResizeRelocateItem_parent parent ) throws InternException
	{
		super( comp, flags, parent );
		_jsp = comp;
		saveStateOfLastDraggedDividerLocation();

		SplitPaneUI spui = comp.getUI();
		if (spui instanceof BasicSplitPaneUI)
		{
			// Setting a mouse listener directly on split pane does not work, because no events are being received.
			((BasicSplitPaneUI) spui).getDivider().addMouseListener(this);
		}
	}

	public void execute()
	{
		Rectangle parentBounds = getParentBounds( _component );
		Rectangle newBounds = calculateNewBounds( parentBounds );
		setBounds(newBounds);
	}

	public void saveStateOfLastDraggedDividerLocation()
	{
		_lastDraggedDividerLocation = _jsp.getDividerLocation();
		_jspSizeOfLastDraggedDividerLocationEvent = _jsp.getSize();
	}
	
	public Rectangle calculateNewBounds( Rectangle parentBounds )
	{
		Rectangle newBounds = super.calculateNewBounds(parentBounds);
		
		int dividerLocation = calculateNewDividerLocation( newBounds );
		
		int excessLeftOrTop = -1;
		int excessRightOrBottom = -1;
		if( _jsp.getOrientation() == JSplitPane.HORIZONTAL_SPLIT )
		{
			excessLeftOrTop = calculateHorizontalExcess( _jsp.getLeftComponent(), calculateBounds_JSplitPane( _jsp, newBounds, dividerLocation, LEFT ) );
			excessRightOrBottom = calculateHorizontalExcess( _jsp.getRightComponent(), calculateBounds_JSplitPane( _jsp, newBounds, dividerLocation, RIGHT ) );
		}
		else if( _jsp.getOrientation() == JSplitPane.VERTICAL_SPLIT )
		{
			excessLeftOrTop = calculateVerticalExcess( _jsp.getTopComponent(), calculateBounds_JSplitPane( _jsp, newBounds, dividerLocation, TOP ) );
			excessRightOrBottom = calculateVerticalExcess( _jsp.getBottomComponent(), calculateBounds_JSplitPane( _jsp, newBounds, dividerLocation, BOTTOM ) );
		}

		_newDividerLocation = dividerLocation;
		if( excessLeftOrTop + excessRightOrBottom < 0 )	// this is the worst case. We cannot do nothing and the position of the dividerLocation will be reset
		{
				
		}
		else if( excessLeftOrTop < 0 )
		{
			_newDividerLocation = dividerLocation - excessLeftOrTop;
		}
		else if( excessRightOrBottom < 0 )
		{
			_newDividerLocation = dividerLocation + excessRightOrBottom;
		}
		
		return( newBounds );
	}

	public int calculateHorizontalExcess( Component comp, Rectangle triedNewBounds )
	{
		int result = (int) ( triedNewBounds.getWidth() - comp.getMinimumSize().getWidth() );
		
		return( result );
	}
	
	public int calculateVerticalExcess( Component comp, Rectangle triedNewBounds )
	{
		int result = (int) ( triedNewBounds.getHeight() - comp.getMinimumSize().getHeight() );
		
		return( result );
	}
	
	public int calculateNewDividerLocation( Rectangle newBounds )
	{
		int result = _lastDraggedDividerLocation;
	
		if( _jsp.getOrientation() == JSplitPane.HORIZONTAL_SPLIT )
		{
			double expansionLeft = _jsp.getResizeWeight() * ( newBounds.getWidth() - _jspSizeOfLastDraggedDividerLocationEvent.getWidth() );
			result = result + (int) Math.round(expansionLeft);
		}
		else if( _jsp.getOrientation() == JSplitPane.VERTICAL_SPLIT )
		{
			double expansionTop = _jsp.getResizeWeight() * ( newBounds.getHeight() - _jspSizeOfLastDraggedDividerLocationEvent.getHeight() );
			result = result + (int) Math.round(expansionTop);
		}

		return( result );
	}

	protected void resizeChangingPreferredSize( Component comp, Rectangle parentBounds )
	{
		if( comp != null )
		{
			ResizeRelocateItem rri = null;
			Dimension newSize = null;
			if( ( _parent != null ) && ( ( rri = _parent.getResizeRelocateComponentItem(comp) ) != null ) )
			{
				Rectangle newBounds = rri.calculateNewBounds( parentBounds );
				newSize = new Dimension( (int) newBounds.getWidth(), (int) newBounds.getHeight() );
			}
			else
			{
				newSize = new Dimension( (int) parentBounds.getWidth(), (int) parentBounds.getHeight() );
			}
			
			Dimension checkedNewSize = new Dimension( (int) Math.max( Math.min( newSize.getWidth(), comp.getMaximumSize().getWidth() ),
																		comp.getMinimumSize().getWidth() ),
														(int) Math.max( Math.min( newSize.getHeight(), comp.getMaximumSize().getHeight() ),
																		comp.getMinimumSize().getHeight() )
													);
			comp.setPreferredSize( checkedNewSize );
		}
	}

	protected void setBounds( Rectangle newBounds )
	{
		if( _jsp.getOrientation() == JSplitPane.HORIZONTAL_SPLIT )
		{
			resizeChangingPreferredSize( _jsp.getLeftComponent(), calculateBounds_JSplitPane( _jsp, newBounds, _newDividerLocation, LEFT ) );
			resizeChangingPreferredSize( _jsp.getRightComponent(), calculateBounds_JSplitPane( _jsp, newBounds, _newDividerLocation, RIGHT ) );
		}
		else if( _jsp.getOrientation() == JSplitPane.VERTICAL_SPLIT )
		{
			resizeChangingPreferredSize( _jsp.getTopComponent(), calculateBounds_JSplitPane( _jsp, newBounds, _newDividerLocation, TOP ) );
			resizeChangingPreferredSize( _jsp.getBottomComponent(), calculateBounds_JSplitPane( _jsp, newBounds, _newDividerLocation, BOTTOM ) );
		}

		_jsp.setBounds(newBounds);
		_jsp.setDividerLocation( _newDividerLocation );
		_jsp.resetToPreferredSizes();
	}
	
	// Five methods from MouseListener:
	/** Called when the mouse has been clicked on a component. */
	public void mouseClicked(MouseEvent e)
	{
	}

	/** Called when the mouse enters a component. */
	public void mouseEntered(MouseEvent e)
	{
		_inDrag = false;
	}

	/** Called when the mouse exits a component. */
	public void mouseExited(MouseEvent e)
	{
		_inDrag = false;
	}

	/** Called when the mouse has been pressed. */
	public void mousePressed(MouseEvent e)
	{
		_inDrag = true;
	}

	/** Called when the mouse has been released. */
	@Override
	public void mouseReleased(MouseEvent e)
	{
		_inDrag = false;
		saveStateOfLastDraggedDividerLocation();
	}
}
