/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.files;

import com.frojasg1.general.files.BinaryFile;
import com.frojasg1.general.files.BinaryFileException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.file.FileSystems;
import java.nio.file.Path;
import java.nio.file.StandardOpenOption;

/**
 *
 * @author Usuario
 */
public class DesktopBinaryFile implements BinaryFile
{
	protected FileChannel _fileChannel = null;
	
	public DesktopBinaryFile()
	{
		
	}

	@Override
	public void openForReading(String fileName) throws BinaryFileException
	{
		try
		{
			Path path = FileSystems.getDefault().getPath(fileName);
			_fileChannel = FileChannel.open( path, StandardOpenOption.READ );
		}
		catch( Throwable th )
		{
			throw( new BinaryFileException( th.toString() ) );
		}
	}

	@Override
	public void openForReadingAndWriting(String fileName, boolean truncateExisting) throws BinaryFileException
	{
		try
		{
			Path path = FileSystems.getDefault().getPath(fileName);
			if( ! truncateExisting )
				_fileChannel = FileChannel.open( path, StandardOpenOption.READ, StandardOpenOption.WRITE, StandardOpenOption.CREATE );
			else
				_fileChannel = FileChannel.open( path, StandardOpenOption.READ, StandardOpenOption.WRITE, StandardOpenOption.CREATE, StandardOpenOption.TRUNCATE_EXISTING );
		}
		catch( Throwable th )
		{
			throw( new BinaryFileException( th.toString() ) );
		}
	}

	@Override
	public void openForReadingAndWriting_SYNC(String fileName, boolean truncateExisting) throws BinaryFileException
	{
		try
		{
			Path path = FileSystems.getDefault().getPath(fileName);
			if( ! truncateExisting )
				_fileChannel = FileChannel.open( path, StandardOpenOption.READ, StandardOpenOption.WRITE, StandardOpenOption.CREATE, StandardOpenOption.SYNC );
			else
				_fileChannel = FileChannel.open( path, StandardOpenOption.READ, StandardOpenOption.WRITE, StandardOpenOption.CREATE,
														StandardOpenOption.SYNC, StandardOpenOption.TRUNCATE_EXISTING );
		}
		catch( Throwable th )
		{
			throw( new BinaryFileException( th.toString() ) );
		}
	}

	@Override
	public void openForReadingAndWriting_DSYNC(String fileName, boolean truncateExisting) throws BinaryFileException
	{
		try
		{
			Path path = FileSystems.getDefault().getPath(fileName);
			if( ! truncateExisting )
				_fileChannel = FileChannel.open( path, StandardOpenOption.READ, StandardOpenOption.WRITE, StandardOpenOption.CREATE, StandardOpenOption.DSYNC );
			else
				_fileChannel = FileChannel.open( path, StandardOpenOption.READ, StandardOpenOption.WRITE, StandardOpenOption.CREATE,
														StandardOpenOption.DSYNC, StandardOpenOption.TRUNCATE_EXISTING );
		}
		catch( Throwable th )
		{
			throw( new BinaryFileException( th.toString() ) );
		}
	}

	@Override
	public int write(ByteBuffer bb) throws BinaryFileException
	{
		int result = -1;
		try
		{
			result = _fileChannel.write( bb );
		}
		catch( RuntimeException rte )
		{
			throw( rte );
		}
		catch( Throwable th )
		{
			throw( new BinaryFileException( th.toString() ) );
		}
		return( result );
	}

	@Override
	public int write(ByteBuffer bb, long position) throws BinaryFileException
	{
		int result = -1;
		try
		{
			result = _fileChannel.write( bb, position );
		}
		catch( RuntimeException rte )
		{
			throw( rte );
		}
		catch( Throwable th )
		{
			throw( new BinaryFileException( th.toString() ) );
		}
		return( result );
	}

	@Override
	public int read(ByteBuffer bb) throws BinaryFileException
	{
		int result = -1;
		try
		{
			result = _fileChannel.read( bb );
		}
		catch( RuntimeException rte )
		{
			throw( rte );
		}
		catch( Throwable th )
		{
			throw( new BinaryFileException( th.toString() ) );
		}
		return( result );
	}

	@Override
	public int read(ByteBuffer bb, long position) throws BinaryFileException
	{
		int result = -1;
		try
		{
			result = _fileChannel.read( bb, position );
		}
		catch( RuntimeException rte )
		{
			throw( rte );
		}
		catch( Throwable th )
		{
			throw( new BinaryFileException( th.toString() ) );
		}
		return( result );
	}

	@Override
	public void close() throws BinaryFileException
	{
		try
		{
			_fileChannel.close();
		}
		catch( RuntimeException rte )
		{
			throw( rte );
		}
		catch( Throwable th )
		{
			throw( new BinaryFileException( th.toString() ) );
		}
	}

}
