/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.files;

import com.frojasg1.general.GenericFileFunctions;
import com.frojasg1.general.StringFunctions;
import com.frojasg1.general.desktop.files.charset.CharsetManager;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.channels.FileChannel;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.Document;
import javax.swing.text.rtf.RTFEditorKit;

/**
 *
 * @author Usuario
 */
public class FileFunctions
{
	public static String  DIR_SEPARATOR = System.getProperty( "file.separator" );

	static public String getExtension( File file )
    {
		return( GenericFileFunctions.getExtension( file ) );
    }

    static public String getExtension( String fileName )
    {
        return( GenericFileFunctions.getExtension( fileName ) );
    }
	
    static public String cutOffExtension( String fileName )
    {
        return( GenericFileFunctions.cutOffExtension(fileName) );
    }

	public static Document loadRtfDocumentFromFile( String fileName )
	{
		Document result = null;

		FileInputStream fis = null;
		InputStreamReader isr = null;
		try
		{
			String charsetName = CharsetManager.instance().M_detectCharset( fileName );

			fis = new FileInputStream( fileName );

			if( charsetName != null )	isr = new InputStreamReader( fis, charsetName );
			else							isr = new InputStreamReader( fis );

			RTFEditorKit ek = new RTFEditorKit();
			Document doc = new DefaultStyledDocument();
			ek.read( isr, doc, 0);

			result = doc;
		}
		catch( Throwable th )
		{
			th.printStackTrace();
			result = null;
		}
		finally
		{
			try
			{
				if( isr != null )
					isr.close();
				if( fis != null )
					fis.close();
			}
			catch( Throwable th )
			{
			}
		}
		return( result );
	}

	public static String convertFolderSeparator( String compoundFolder )
	{
		return( GenericFileFunctions.convertFolderSeparator(compoundFolder) );
	}

	public static boolean isDirectory( String dirname )
	{
		return( GenericFileFunctions.isDirectory(dirname) );
	}

	public static boolean isFile( String fileName )
	{
		return( GenericFileFunctions.isFile(fileName) );
	}

	// http://stackoverflow.com/questions/106770/standard-concise-way-to-copy-a-file-in-java
	public static void copyFile( String sourceFileName, String destFileName ) throws IOException
	{
		GenericFileFunctions.copyFile(sourceFileName, destFileName);
	}

	public static void copyDirectoryRecursive( String oldDir, String newDir ) throws IOException
	{
		GenericFileFunctions.copyDirectoryRecursive(oldDir, newDir);
	}

	public static void eraseRecursive( String dir, String fileToErase ) throws IOException
	{
		GenericFileFunctions.eraseRecursive(dir, fileToErase);
	}

	public static String getTextContentsOfAFile( String fileName, String charsetName ) throws IOException
	{
		return( GenericFileFunctions.getTextContentsOfAFile(fileName, charsetName) );
	}
}
