/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.textcrypt.controller.threads;

import com.frojasg1.general.progress.CancellationException;
import com.frojasg1.general.progress.OperationCancellation;
import com.frojasg1.textcrypt.controller.TextcryptController;
import com.hotmail.frojasg1.applications.fileencoder.FileEncoder;
import com.hotmail.frojasg1.applications.fileencoder.FileEncoderException;
import com.hotmail.frojasg1.applications.fileencoder.FileEncoderParameters;
import java.io.File;
import java.nio.file.Files;
import java.nio.file.StandardCopyOption;

/**
 *
 * @author axpe
 */
public class SaveEncryptedTextThread extends ParentThread
{
	protected String _fileName = null;
	protected char[] _password = null;
	protected byte[] _dataToEncode = null;
	protected int _strength = 256;
	
	public SaveEncryptedTextThread( TextcryptController controller,
									String fileName,
									char[] password,
									int strength,
									String text,
									OperationCancellation oc )
	{
		super( controller, oc );
		_fileName = fileName;
		_password =  password;
		_strength = strength;
		try
		{
			_dataToEncode = text.getBytes( "UTF8" );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
			_dataToEncode = text.getBytes();
		}
	}

	@Override
    public void run()
    {
		try
		{
			File file = new File( _fileName );
			if( file.exists() )
			{
				File oldFile = new File( _fileName + "__old__" );
				try
				{
					Files.copy(file.toPath(), oldFile.toPath(), StandardCopyOption.REPLACE_EXISTING);
				}
				catch( Throwable th )
				{
					th.printStackTrace();
					throw( new FileEncoderException( "Problem while saving old file. We abort the operation." ) );
				}
			}

//			FileEncoderType_3 fe = new FileEncoderType_3( );

			FileEncoderParameters fep = new FileEncoderParameters();
			fep.M_setFileEncoderType( FileEncoderParameters.INT_FILE_ENCODER_TYPE_4 );
			fep.M_setSliceSize( 256000 );
			fep.M_setNumberOfBitsPerIterationReordererEncoder( 1 );
			fep.M_setNumberOfBitsPerIterationSimpleEncoder( 1 );
			fep.M_setSizeOfNumbersReordererEncoder( _strength );
			fep.M_setSizeOfNumbersSimpleEncoder( _strength );

			FileEncoder.M_encodeFileFromBuffer( _password, fep,
												_dataToEncode,
												_fileName,
												_controller.getView(),
												a_oc );
//			fe.M_encodeFile( _password, fep, _dataToEncode, _fileName, _controller.getView(), a_oc );
		}
		catch( FileEncoderException ex )
		{
			M_setErrorString( ex.getMessage() );
		}
		catch (CancellationException ex)
		{
			M_setIsCancellation( true );
		}
		catch( Throwable th )
		{
			M_setErrorString( th.getMessage() );
		}

		if( M_getItWasError() )
        {
            _controller.textCrypt_fileError( M_getErrorString() );
        }
        else if( M_isCancellation() )
		{
			_controller.textCrypt_fileCancelled();
		}
		else
			_controller.textCrypt_fileSaved();

        M_setHasEnded(true);
    }
}
